module aplot
use ISO_C_BINDING
implicit none

    private::pass_string_to_plot

    integer, parameter::APLOT_STYLE_LINE = 0
    integer, parameter::APLOT_STYLE_DOT = 1
    integer, parameter::APLOT_STYLE_BAR = 2
    integer, parameter::APLOT_STYLE_PIXEL = 3
    
    type aplot_t
        type(c_ptr)::p
    end type aplot_t
    
    interface
        function c_initialize_plot() bind(c, name="initialize_plot")
        use iso_c_binding
        type(c_ptr)::c_initialize_plot
        end function c_initialize_plot
    end interface

    interface
        subroutine c_destroy_plot(p) bind(c, name="destroy_plot")
        use iso_c_binding
        type(c_ptr), value::p
        end subroutine c_destroy_plot
    end interface
    
    interface
        subroutine c_display_plot(p) bind(c, name="display_plot")
        use iso_c_binding
        type(c_ptr), value::p
        end subroutine c_display_plot
    end interface
    
    interface
        subroutine c_add_dataset(p, x, y, n) bind(c, name="add_dataset")
        use iso_c_binding
        type(c_ptr), value::p
        real(kind=c_double), dimension(*)::x, y
        integer(kind=c_int), value::n
        end subroutine c_add_dataset
    end interface
    
    interface
        subroutine c_set_title(p, nm) bind(c, name="set_title")
        use iso_c_binding
        type(c_ptr), value::p
        character(kind=c_char), dimension(*)::nm
        end subroutine c_set_title
    end interface
    
    interface
        subroutine c_set_xlabel(p, nm) bind(c, name="set_xlabel")
        use iso_c_binding
        type(c_ptr), value::p
        character(kind=c_char), dimension(*)::nm
        end subroutine c_set_xlabel
    end interface
    
    interface
        subroutine c_set_ylabel(p, nm) bind(c, name="set_ylabel")
        use iso_c_binding
        type(c_ptr), value::p
        character(kind=c_char), dimension(*)::nm
        end subroutine c_set_ylabel
    end interface
    
    interface
        subroutine c_set_xscale(p, vmin, vmax) bind(c, name="set_xscale")
        use ISO_C_BINDING
        type(c_ptr), value::p
        real(kind=c_double), value::vmin, vmax
        end subroutine c_set_xscale
    end interface
    
    interface
        subroutine c_set_yscale(p, vmin, vmax) bind(c, name="set_yscale")
        use ISO_C_BINDING
        type(c_ptr), value::p
        real(kind=c_double), value::vmin, vmax
        end subroutine c_set_yscale
    end interface
    
    interface
        subroutine c_set_xautoscale(p) bind(c, name="set_xautoscale")
        use ISO_C_BINDING
        type(c_ptr), value::p
        end subroutine c_set_xautoscale
    end interface
    
    interface
        subroutine c_set_yautoscale(p) bind(c, name="set_yautoscale")
        use ISO_C_BINDING
        type(c_ptr), value::p
        end subroutine c_set_yautoscale
    end interface
    
    interface
        subroutine c_set_serieslabel(p, i, nm) bind(c, name="set_serieslabel")
        use iso_c_binding
        type(c_ptr), value::p
        integer(kind=c_int), value::i
        character(kind=c_char), dimension(*)::nm
        end subroutine c_set_serieslabel
    end interface
    
    interface
        subroutine c_set_seriestype(p, i, tp) bind(c, name="set_seriestype")
        use ISO_C_BINDING
        implicit none
        type(c_ptr), value::p
        integer(kind=c_int), value::i,tp
        end subroutine c_set_seriestype
    end interface
    
    interface add_dataset
        module procedure add_dataset_4
        module procedure add_dataset_8
    end interface
    
    interface 
        subroutine c_set_xlinear(p) bind(c, name="set_xlinear")
        use ISO_C_BINDING
        type(c_ptr), value::p
        end subroutine c_set_xlinear
        
        subroutine c_set_ylinear(p) bind(c, name="set_ylinear")
        use ISO_C_BINDING
        type(c_ptr), value::p
        end subroutine c_set_ylinear
    end interface
    
    interface 
        subroutine c_set_xlogarithmic(p, base) bind(c, name="set_xlogarithmic")
        use ISO_C_BINDING
        type(c_ptr), value::p
        real(kind=c_float), value::base
        end subroutine c_set_xlogarithmic
        
        subroutine c_set_ylogarithmic(p, base) bind(c, name="set_ylogarithmic")
        use ISO_C_BINDING
        type(c_ptr), value::p
        real(kind=c_float), value::base
        end subroutine c_set_ylogarithmic
    end interface
    
contains

    function initialize_plot()
    implicit none
    
        type(aplot_t)::initialize_plot
        
        initialize_plot%p = c_initialize_plot();
        
    end function initialize_plot
    
    subroutine destroy_plot(p)
    use ISO_C_BINDING, only: C_NULL_PTR
    implicit none
    
        type(aplot_t), intent(inout)::p
        
        call c_destroy_plot(p%p)
        p%p = C_NULL_PTR
        
    end subroutine destroy_plot
    
    subroutine display_plot(p)
    implicit none
    
        type(aplot_t), intent(in)::p
        
        call c_display_plot(p%p)
        
    end subroutine display_plot
    
    subroutine add_dataset_8(p, x, y)
    implicit none
    
        type(aplot_t)::p
        real(kind=8), dimension(:), intent(in)::x, y
        integer:: n

        n = MIN(size(x), size(y))
        
        call c_add_dataset(p%p, x, y, n)
        
    end subroutine add_dataset_8
    
    subroutine add_dataset_4(p, x, y)
    use iso_c_binding, only: c_double
    implicit none
    
        type(aplot_t)::p
        real(kind=4), dimension(:), intent(in)::x, y
        integer:: n
        real(kind=c_double), dimension(:), allocatable::xpass, ypass
        
        allocate(xpass(size(x)))
        allocate(ypass(size(y)))
    
        xpass = real(x, kind=c_double)
        ypass = real(y, kind=c_double)
        n = MIN(size(x), size(y))
        call c_add_dataset(p%p, xpass, ypass, n)
        
    end subroutine add_dataset_4
    
    subroutine pass_string_to_plot(p, nm, f)
    use ISO_C_BINDING, only: c_char, C_NULL_CHAR
    implicit none
    
        type(aplot_t)::p
        character(*), intent(in)::nm
        interface
            subroutine f(p, nm)
            use ISO_C_BINDING, only: c_ptr, c_char
            type(c_ptr), value::p
            character(kind=c_char), dimension(*)::nm
            end subroutine f
        end interface
        
        character(kind=c_char), dimension(:), allocatable::npass
        integer::i
        
        allocate(npass(len_trim(nm)+1))
        
        npass = C_NULL_CHAR
        
        do i=1, len_trim(nm)
            npass(i) = nm(i:i)
        end do
        
        call f(p%p, npass)
        
    end subroutine pass_string_to_plot
    
    subroutine set_title(p, nm)
    implicit none
    
        type(aplot_t)::p
        character(*), intent(in)::nm
        
        call pass_string_to_plot(p, nm, c_set_title)
        
    end subroutine set_title 
    
    subroutine set_xlabel(p, nm)
    implicit none
    
        type(aplot_t)::p
        character(*), intent(in)::nm
        
        call pass_string_to_plot(p, nm, c_set_xlabel)
        
    end subroutine set_xlabel
    
    subroutine set_ylabel(p, nm)
    implicit none
    
        type(aplot_t)::p
        character(*), intent(in)::nm
        
        call pass_string_to_plot(p, nm, c_set_ylabel)
        
    end subroutine set_ylabel
    
    subroutine set_xscale(p, vmin, vmax)
    use ISO_C_BINDING, only: c_double
    implicit none
        type(aplot_t)::p
        real, intent(in)::vmin, vmax
        
        call c_set_xscale(p%p, real(vmin, kind=c_double), real(vmax, kind=c_double))
        
    end subroutine set_xscale
    
    subroutine set_yscale(p, vmin, vmax)
    use ISO_C_BINDING, only: c_double
    implicit none
        type(aplot_t)::p
        real, intent(in)::vmin, vmax
        
        call c_set_yscale(p%p, real(vmin, kind=c_double), real(vmax, kind=c_double))
        
    end subroutine set_yscale
    
    subroutine set_xautoscale(p)
    implicit none
        type(aplot_t)::p
        
        call c_set_xautoscale(p%p)
        
    end subroutine set_xautoscale
    
    subroutine set_yautoscale(p)
    implicit none
        type(aplot_t)::p
        
        call c_set_yautoscale(p%p)
        
    end subroutine set_yautoscale
    
    subroutine set_xlinear(p)
    implicit none
        type(aplot_t)::p
        
        call c_set_xlinear(p%p)
        
    end subroutine set_xlinear
    
    subroutine set_ylinear(p)
    implicit none
        type(aplot_t)::p
        
        call c_set_ylinear(p%p)
        
    end subroutine set_ylinear
    
    subroutine set_xlogarithmic(p, base)
    use ISO_C_BINDING
    implicit none
        type(aplot_t)::p
        real, optional::base
        real(kind=c_float)::internal_base
        
        internal_base = 10.0
        if(present(base)) then
            internal_base = base
        end if
        
        call c_set_xlogarithmic(p%p, internal_base)
        
    end subroutine set_xlogarithmic
    
    subroutine set_ylogarithmic(p, base)
    use ISO_C_BINDING
    implicit none
        type(aplot_t)::p
        real, optional::base
        real(kind=c_float)::internal_base
        
        internal_base = 10.0
        if(present(base)) then
            internal_base = base
        end if
        
        call c_set_ylogarithmic(p%p, internal_base)
        
    end subroutine set_ylogarithmic
    
    subroutine set_seriestype(p, i, tp)
    implicit none
        
        type(aplot_t)::p
        integer, intent(in)::i, tp
        
        call  c_set_seriestype(p%p, i, tp)
        
    end subroutine set_seriestype
    
    subroutine set_serieslabel(p, i, nm)
    implicit none
        
        type(aplot_t)::p
        integer, intent(in)::i
        character(*), intent(in)::nm
        
        character(kind=c_char), dimension(:), allocatable::npass
        integer::k
        
        allocate(npass(len_trim(nm)+1))
        
        npass = C_NULL_CHAR
        
        do k=1, len_trim(nm)
            npass(k) = nm(k:k)
        end do
        
        call c_set_serieslabel(p%p, i, npass)
        
    end subroutine set_serieslabel
    
end module aplot
